@@ -4,6 +4,7 @@
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
  * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  * Copyright (C) 2022-2023 Antenore Gatta, Giovanni Panozzo, Hiroyuki Tanaka
+ * Copyright (C) 2023-2024 Hiroyuki Tanaka, Sunil Bhat
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,6 +53,7 @@ typedef struct _RemminaMain {
 	GtkMenu *		menu_popup;
 	GtkMenuButton *		menu_header_button;
 	GtkMenu *		menu_popup_full;
+	GtkMenu *       menu_popup_multi;
 	GtkRadioMenuItem *	menuitem_view_mode_list;
 	GtkRadioMenuItem *	menuitem_view_mode_tree;
 	GtkMenuItem *		menuitem_connection_quit;
@@ -83,6 +85,7 @@ typedef struct _RemminaMain {
 	GtkAccelGroup *		accelgroup_shortcuts;
 	RemminaMainPriv *	priv;
 	RemminaMonitor *	monitor;
+	GHashTable *		network_states;
 } RemminaMain;
 
 struct _RemminaMainPriv {
@@ -106,14 +109,17 @@ GtkWidget *remmina_main_new(void);
 GtkWindow *remmina_main_get_window(void);
 
 void remmina_main_update_file_datetime(RemminaFile *file);
+void remmina_main_add_network_status(gchar* key, gchar* value);
 
 void remmina_main_destroy(void);
 void remmina_main_on_destroy_event(void);
+gboolean remmina_main_on_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);
 void remmina_main_save_before_destroy(void);
 
 void remmina_main_show_dialog(GtkMessageType msg, GtkButtonsType buttons, const gchar* message);
 void remmina_main_show_warning_dialog(const gchar *message);
 void remmina_main_on_action_application_about(GSimpleAction *action, GVariant *param, gpointer data);
+void remmina_main_on_action_application_bug_report(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_application_default(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_application_mpchange(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_application_plugins(GSimpleAction *action, GVariant *param, gpointer data);
@@ -123,6 +129,8 @@ void remmina_main_on_action_application_quit(GSimpleAction *action, GVariant *pa
 void remmina_main_on_action_connection_connect(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_connection_copy(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_connection_delete(GSimpleAction *action, GVariant *param, gpointer data);
+void remmina_main_on_action_connection_delete_multiple(GSimpleAction *action, GVariant *param, gpointer data);
+void remmina_main_on_action_connection_connect_multiple(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_connection_edit(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_connection_external_tools(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_connection_new(GSimpleAction *action, GVariant *param, gpointer data);
@@ -136,6 +144,22 @@ void remmina_main_on_action_tools_import(GSimpleAction *action, GVariant *param,
 void remmina_main_on_action_expand(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_collapse(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_search_toggle(GSimpleAction *action, GVariant *param, gpointer data);
-void remmina_main_reload_preferences();
+void remmina_main_toggle_password_view(GtkWidget *widget, gpointer data);
+void remmina_main_reload_preferences(void);
+
+void remmina_main_quick_search_enter(GtkWidget *widget, gpointer user_data);
+void remmina_main_quick_search_on_icon_press(GtkEntry *entry, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data);
+void remmina_main_quick_search_on_changed(GtkEditable *editable, gpointer user_data);
+void remmina_main_on_view_toggle(void);
+void remmina_main_on_date_column_sort_clicked(void);
+void remmina_main_file_list_on_row_activated(GtkTreeView *tree, GtkTreePath *path, GtkTreeViewColumn *column, gpointer user_data);
+gboolean remmina_main_file_list_on_key_press(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
+gboolean remmina_main_file_list_on_button_press(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
+void remmina_main_load_files_cb(GtkEntry *entry, char *string, gpointer user_data);
+gboolean remmina_main_quickconnect_on_click(GtkWidget *widget, gpointer user_data);
+gboolean remmina_main_on_window_state_event(GtkWidget *widget, GdkEventWindowState *event, gpointer user_data);
+void remmina_main_on_show(GtkWidget *w, gpointer user_data);
+void remmina_main_on_drag_data_received(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y,
+					GtkSelectionData *data, guint info, guint time, gpointer user_data);
 
 G_END_DECLS
